@@ -4,7 +4,7 @@
 #define VERSION "x.x"
 
 /* for runtime debugging */
-#undef GNP_DEBUG
+#undef APP_DEBUG
 
 /* for file locking */
 #undef USE_FCNTL
@@ -12,3 +12,6 @@
 
 /* for project/ide features (experimental) */
 #undef WANT_PROJECT
+
+/* for gnotepad+ ! */
+#define APP_GNP	1
