@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: debug.c,v 1.1 1998/10/08 18:54:46 kahn Exp kahn $
+ * $Id: debug.c,v 1.2 1999/01/18 19:43:21 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -21,13 +21,13 @@
 #include "main.h"
 #include "debug.h"
 
-#ifdef GNP_DEBUG
+#ifdef APP_DEBUG
 
 long dbg_flags = 0;
 
 /* eventually, there may be more debugging routines in here */
 /* for now, the printf macros in debug.h are enough */
 
-#endif	/* GNP_DEBUG */
+#endif	/* APP_DEBUG */
 
 /* the end */
