@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: doc.c,v 1.18 1998/10/16 20:15:52 kahn Exp kahn $
+ * $Id: doc.c,v 1.34 1999/01/21 21:10:12 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
+#include "main.h"
 #include <gdk/gdkkeysyms.h>
 #include "doc.h"
 #include "menu.h"
@@ -35,7 +36,10 @@
 #include "msgbox.h"
 #include "prefs.h"
 #include "recent.h"
+#ifdef APP_GNP
 #include "project.h"
+#include "print.h"
+#endif
 
 /*** external functions ***/
 extern void win_set_title(doc_t *);
@@ -48,40 +52,60 @@ extern void win_set_title(doc_t *);
 
 typedef struct {
 	GtkWidget *toplev;
-	GList *adv_list;
+	GSList *adv_list;
 	GtkWidget *adv_button;
 } doc_info_t;
 
 
 /*** local variables ***/
 static unsigned nextdocid = 0;
+static toolbar_data_t dlw_tbdata[] = {
+#ifdef APP_GNP
+	{ " Save ",	"Save file",		"dlw Save",	"tb_save.xpm",
+			(GtkSignalFunc)doc_save_cb },
+#endif
+	{ " Close ",	"Close current file",	"dlw Close",	"tb_cancel.xpm",
+			(GtkSignalFunc)doc_close_cb },
+#ifdef APP_GNP
+	{ " Print ",	"Print file",		"dlw Print",	"tb_print.xpm",
+			(GtkSignalFunc)print_cb },
+#endif
+	{ " SPACE " },
+	{ " Ok ",	"Close list window",	"dlw Ok",	"tb_exit.xpm",
+			(GtkSignalFunc)doc_list_destroy },
+	{ NULL }
+};
 
 
 /*** local function prototypes ***/
-static void doc_add_to_notebook(doc_t *d);
+static void	doc_add_to_notebook(doc_t *d);
 #ifdef USE_AUTO_INDENT
 static gboolean doc_auto_indent_cb(GtkWidget *, GdkEventKey *, win_t *w);
 #endif
-static bool_t doc_close_common(win_t *w);
-static bool_t doc_close_execute(win_t *w);
-static bool_t doc_close_verify_yes(win_t *w);
-static bool_t doc_close_verify(win_t *w);
-static void doc_free(doc_t *);
-static void doc_info_advanced(GtkWidget *wgt, gpointer cbdata);
-static void doc_info_close(GtkWidget *wgt, gpointer cbdata);
-static void doc_info_destroy(GtkWidget *wgt, gpointer cbdata);
-static void doc_info_table_add(GtkWidget *, doc_info_t *,
-		char *, int , int , int , int, bool_t);
-static void doc_list_select(GtkWidget *wgt, int row, int column,
-		GdkEventButton *event, gpointer cbdata);
-static void doc_list_update(win_t *w, doc_t *d, int rownum, char *text);
-static void doc_open_filesel_destroy(GtkWidget *wgt, gpointer cbdata);
-static void doc_open_ok(GtkWidget *wgt, gpointer cbdata);
-static doc_t * doc_remove(win_t *w);
-static void doc_saveas_destroy(GtkWidget *wgt, gpointer cbdata);
-static void doc_saveas_ok(GtkWidget *wgt, gpointer data);
-static void doc_open_dialog_create(win_t *w, char *title,
-		void (*open_ok)(GtkWidget *, gpointer));
+static bool_t	doc_close_common(win_t *w);
+static bool_t	doc_close_execute(win_t *w);
+#ifdef APP_GNP
+static bool_t	doc_close_verify_yes(win_t *w);
+static bool_t	doc_close_verify(win_t *w);
+#endif
+static void	doc_free(doc_t *);
+static void	doc_info_advanced(GtkWidget *wgt, gpointer cbdata);
+static void	doc_info_close(GtkWidget *wgt, gpointer cbdata);
+static void	doc_info_destroy(GtkWidget *wgt, gpointer cbdata);
+static void	doc_info_table_add(GtkWidget *, doc_info_t *, char *, int, int,
+				   int , int, bool_t);
+static int	doc_list_select(GtkWidget *, int, int, GdkEventButton *,
+				gpointer);
+static void	doc_list_update(win_t *w, doc_t *d, int rownum, char *text);
+static void	doc_open_filesel_destroy(GtkWidget *wgt, gpointer cbdata);
+static void	doc_open_ok(GtkWidget *wgt, gpointer cbdata);
+static doc_t *	doc_remove(win_t *w);
+#ifdef APP_GNP
+static void	doc_saveas_destroy(GtkWidget *wgt, gpointer cbdata);
+static void	doc_saveas_ok(GtkWidget *wgt, gpointer data);
+#endif
+static void	doc_open_dialog_create(win_t *w, char *title,
+				       void (*open_ok)(GtkWidget *, gpointer));
 
 
 /*** global function definitions ***/
@@ -107,7 +131,7 @@ doc_basefname(doc_t *d)
 void
 doc_by_name(GtkWidget *wgt, gpointer cbdata)
 {
-	GList *dlp;
+	GSList *dlp;
 	char *full;
 	int num;
 	doc_t *d;
@@ -131,8 +155,7 @@ doc_by_name(GtkWidget *wgt, gpointer cbdata)
 
 		if (strcmp(full, dnnp->fname) == 0) {
 			g_free(full);
-			gtk_notebook_set_page(
-				GTK_NOTEBOOK(dnnp->w->notebook), num);
+			gtk_notebook_set_page(GTK_NOTEBOOK(dnnp->w->nb), num);
 			break;
 		}
 
@@ -144,16 +167,20 @@ doc_by_name(GtkWidget *wgt, gpointer cbdata)
 	/* not open, so open it, and close 'Untitled' if necessary */
 	if (dlp == NULL) {
 		d = doc_current(dnnp->w);
-		if (dnnp->w->numdoc == 1 && !d->changed &&
-				strcmp(d->fname, UNTITLED) == 0) {
+		if (dnnp->w->numdoc == 1 &&
+#ifdef APP_GNP
+		    !d->changed &&
+#endif
+		    strcmp(d->fname, UNTITLED) == 0) {
 			doc_close_common(dnnp->w);
 		}
 
-		doc_new(dnnp->w, dnnp->fname);
+		doc_new(dnnp->w, dnnp->fname, TRUE);
 	}
 } /* doc_by_name */
 
 
+#ifdef APP_GNP
 /*
  * PUBLIC: doc_changed_cb
  *
@@ -161,13 +188,14 @@ doc_by_name(GtkWidget *wgt, gpointer cbdata)
  * since after saving a file, the document is no longer "changed".
  */
 void
-doc_changed_cb(GtkWidget *w, gpointer cbdata)
+doc_changed_cb(GtkWidget *wgt, gpointer cbdata)
 {
 	doc_t *d = (doc_t *)cbdata;
 
 	d->changed = TRUE;
-	gtk_signal_disconnect(GTK_OBJECT(d->txt), d->changed_id);
+	gtk_signal_disconnect(GTK_OBJECT(d->data), d->changed_id);
 	d->changed_id = FALSE;
+	doc_info_label_update(d->w);
 } /* doc_changed_cb */
 
 
@@ -180,7 +208,7 @@ doc_changed_cb(GtkWidget *w, gpointer cbdata)
 bool_t
 doc_check_if_any_changed(win_t *w)
 {
-	GList *dp;
+	GSList *dp;
 	doc_t *d;
 	bool_t ret = FALSE;
 
@@ -199,6 +227,7 @@ doc_check_if_any_changed(win_t *w)
 
 	return ret;
 } /* doc_check_if_any_changed */
+#endif
 
 
 /*
@@ -217,7 +246,9 @@ doc_close_cb(GtkWidget *wgt, gpointer cbdata)
 	g_assert(d != NULL);
 
 	/* if all we have is a blank, Untitled doc, then return immediately */
+#ifdef APP_GNP
 	if (!d->changed && w->numdoc == 1 && strcmp(d->fname, UNTITLED) == 0)
+#endif
 		return;
 
 	(void)doc_close_common(w);
@@ -225,7 +256,7 @@ doc_close_cb(GtkWidget *wgt, gpointer cbdata)
 	recent_list_write(w);
 
 	if (w->numdoc < 1)
-		doc_new(w, UNTITLED);
+		doc_new(w, UNTITLED, TRUE);
 } /* doc_close_cb */
 
 
@@ -245,7 +276,9 @@ doc_close_all_cb(GtkWidget *wgt, gpointer cbdata)
 	g_assert(d != NULL);
 
 	/* if all we have is a blank, Untitled doc, then return immediately */
+#ifdef APP_GNP
 	if (!d->changed && w->numdoc == 1 && strcmp(d->fname, UNTITLED) == 0)
+#endif
 		return;
 
 	(void)doc_close_all_common(w);
@@ -255,7 +288,7 @@ doc_close_all_cb(GtkWidget *wgt, gpointer cbdata)
 	 * do it here.
 	 */
 	if (w->numdoc < 1)
-		doc_new(w, UNTITLED);
+		doc_new(w, UNTITLED, TRUE);
 } /* doc_close_all_cb */
 
 
@@ -279,18 +312,19 @@ doc_close_all_common(win_t *w)
 		 * to hide the window, and then unhide it again when getting
 		 * to a changed document.
 		 */
-		if (doc_check_if_any_changed(w) == FALSE) {
-			if (GTK_WIDGET_VISIBLE(w->notebook))
-				gtk_widget_hide(w->notebook);
-		}
+#ifdef APP_GNP
+		if (doc_check_if_any_changed(w) == FALSE)
+#endif
+			if (GTK_WIDGET_VISIBLE(w->nb))
+				gtk_widget_hide(w->nb);
 
 		allclosed = doc_close_common(w);
 		if (allclosed == FALSE)	/* not closed */
 			break;
 	}
 
-	if (!GTK_WIDGET_VISIBLE(w->notebook))
-		gtk_widget_show(w->notebook);
+	if (!GTK_WIDGET_VISIBLE(w->nb))
+		gtk_widget_show(w->nb);
 
 	recent_list_write(w);
 
@@ -311,6 +345,25 @@ doc_current(win_t *w)
 } /* doc_current */
 
 
+/*
+ * PUBLIC: doc_foreach
+ *
+ * given a window, traverses all documents of that window and calls func()
+ * on each.
+ */
+void
+doc_foreach(win_t *wp, void (*func)(void *))
+{
+	GSList *dlp;
+	doc_t *dp;
+
+	for (dlp = wp->doclist; dlp; dlp = dlp->next) {
+		dp = (doc_t *)dlp->data;
+		func(dp);
+	}
+} /* doc_foreach */
+
+
 /*
  * PUBLIC: doc_info_cb
  *
@@ -326,7 +379,7 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 	win_t *w = (win_t *)cbdata;
 
 	g_assert(w != NULL);
-	d = (doc_t *)(w->curdoc);
+	d = doc_current(w);
 	g_assert(d != NULL);
 	di = (doc_info_t *)(d->docinfo);
 	if (di) {
@@ -338,7 +391,7 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 	d->docinfo = di;
 	di->toplev = gtk_dialog_new();
 	gtk_signal_connect(GTK_OBJECT(di->toplev), "destroy",
-		GTK_SIGNAL_FUNC(doc_info_destroy), d);
+			   GTK_SIGNAL_FUNC(doc_info_destroy), d);
 	gtk_window_set_title(GTK_WINDOW(di->toplev), "File Information");
 
 	vbox = GTK_DIALOG(di->toplev)->vbox;
@@ -361,14 +414,16 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 
 	doc_info_table_add(table, di, doc_basefname(d),   1, 2, 0, 1, FALSE);
 	doc_info_table_add(table, di, "                ", 1, 2, 1, 2, FALSE);
+#ifdef APP_GNP
 	doc_info_table_add(table, di,
 		(d->changed) ? "Yes" : "No",              1, 2, 2, 3, FALSE);
+#endif
 	doc_info_table_add(table, di, (d->sb) ?
 		((file_is_readonly(d->fname)) ?
 		"Yes" : "No" ) : "No",                    1, 2, 3, 4, FALSE);
 	doc_info_table_add(table, di, " ",                1, 2, 4, 5, FALSE);
 	doc_info_table_add(table, di,
-		ltoa((long)gtk_text_get_length(GTK_TEXT(d->txt))),
+		ltoa((long)gtk_text_get_length(GTK_TEXT(d->data))),
 		1, 2, 5, 6, FALSE);
 	doc_info_table_add(table, di,
 		(d->sb) ? ltoa((long)d->sb->st_size) : UNKNOWN,
@@ -376,7 +431,7 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 
 	tmp = gtk_hseparator_new();
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 0, 2, 7, 8);
-	di->adv_list = g_list_append(di->adv_list, tmp);
+	di->adv_list = g_slist_prepend(di->adv_list, tmp);
 
 	doc_info_table_add(table, di, "Full pathname:",     0, 1, 8, 9, TRUE);
 	doc_info_table_add(table, di, "Inode:",             0, 1, 9, 10, TRUE);
@@ -395,25 +450,25 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 	g_free(s);
 	if (d->sb) {
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_ino),
-			1, 2, 9, 10, TRUE);
+				   1, 2, 9, 10, TRUE);
 		doc_info_table_add(table, di, file_perm_string(d->sb),
-			1, 2, 10, 11, TRUE);
+				   1, 2, 10, 11, TRUE);
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_nlink),
-			1, 2, 11, 12, TRUE);
+				   1, 2, 11, 12, TRUE);
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_uid),
-			1, 2, 12, 13, TRUE);
+				   1, 2, 12, 13, TRUE);
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_gid),
-			1, 2, 13, 14, TRUE);
+				   1, 2, 13, 14, TRUE);
 		doc_info_table_add(table, di, ttoa(d->sb->st_atime, FALSE),
-			1, 2, 14, 15, TRUE);
+				   1, 2, 14, 15, TRUE);
 		doc_info_table_add(table, di, ttoa(d->sb->st_mtime, FALSE),
-			1, 2, 15, 16, TRUE);
+				   1, 2, 15, 16, TRUE);
 		doc_info_table_add(table, di, ttoa(d->sb->st_ctime, FALSE),
-			1, 2, 16, 17, TRUE);
+				   1, 2, 16, 17, TRUE);
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_blksize),
-			1, 2, 17, 18, TRUE);
+				   1, 2, 17, 18, TRUE);
 		doc_info_table_add(table, di, ltoa((long)d->sb->st_blocks),
-			1, 2, 18, 19, TRUE);
+				   1, 2, 18, 19, TRUE);
 	} else {
 		int i;
 		for (i = 9; i <= 18; i++)
@@ -424,15 +479,15 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 	/* buttons */
 	di->adv_button = gtk_button_new_with_label("Advanced");
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(di->toplev)->action_area),
-		di->adv_button, TRUE, TRUE, 0);
+			   di->adv_button, TRUE, TRUE, 0);
 	gtk_signal_connect(GTK_OBJECT(di->adv_button), "clicked",
-		GTK_SIGNAL_FUNC(doc_info_advanced), di);
+			   GTK_SIGNAL_FUNC(doc_info_advanced), di);
 
 	tmp = gtk_button_new_with_label(BUTTON_CLOSE);
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(di->toplev)->action_area),
-		tmp, TRUE, TRUE, 0);
+			   tmp, TRUE, TRUE, 0);
 	gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		GTK_SIGNAL_FUNC(doc_info_close), d);
+			   GTK_SIGNAL_FUNC(doc_info_close), d);
 	GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default(tmp);
 
@@ -449,17 +504,61 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 void
 doc_info_init(win_t *w, GtkWidget *parent)
 {
+	w->docinfo = gtk_button_new();
+	w->docinfo_label = gtk_label_new(" File Info ");
+	gtk_misc_set_alignment(GTK_MISC(w->docinfo_label), 0.5, 0.5);
+	gtk_container_add(GTK_CONTAINER(w->docinfo), w->docinfo_label);
+#if 0
 	w->docinfo = gtk_button_new_with_label("File Info");
+#endif
+
 	gtk_box_pack_end(GTK_BOX(parent), w->docinfo, FALSE, FALSE, 0);
 	gtk_signal_connect(GTK_OBJECT(w->docinfo), "clicked",
 			GTK_SIGNAL_FUNC(doc_info_cb), w);
 
 	/* set according to preferences */
 	if (IS_SHOW_MSGBAR())
-		gtk_widget_show(w->docinfo);
+		gtk_widget_show_all(w->docinfo);
+
 } /* docinfo_init */
 
 
+/*
+ * PUBLIC: doc_info_label_update
+ *
+ * updates the file info button's label to reflect readonly and modified status
+ * of the current document.
+ */
+void
+doc_info_label_update(win_t *w)
+{
+	doc_t *d;
+	char *buf;
+	bool_t ro;
+
+	d = doc_current(w);
+	buf = (char *)g_malloc(strlen(" File Info ") + 7);
+	strcpy(buf, " File Info ");
+
+	ro = file_is_readonly(d->fname);
+
+#ifdef APP_GNP
+	if (d->changed || ro) {
+		strcat(buf, " (");
+		if (d->changed)
+			strcat(buf, "*");
+		if (ro)
+			strcat(buf, "R");
+		strcat(buf, ")");
+#endif
+		strcat(buf, ")");
+	}
+
+	gtk_label_set(GTK_LABEL(w->docinfo_label), buf);
+	g_free(buf);
+} /* doc_info_label_update */
+
+
 /*
  * PUBLIC: doc_list_add
  *
@@ -552,8 +651,10 @@ doc_list_destroy(GtkWidget *wgt, gpointer cbdata)
 		gtk_widget_destroy(w->dlw);
 		w->dlw = NULL;
 		w->dlw_data = NULL;
-		w->dlw_toolbar = NULL;
-		w->dlw_toolbar_h = NULL;
+		w->dlw_tb = NULL;
+#ifndef GTK_HAVE_FEATURES_1_1_0
+		w->dlw_tb_h = NULL;
+#endif
 	}
 } /* doc_list_destroy */
 
@@ -588,7 +689,7 @@ doc_list_remove(win_t *w, int num)
 	}
 
 	/* highlight the current document in the files list window */
-	num = gtk_notebook_current_page(GTK_NOTEBOOK(w->notebook));
+	num = gtk_notebook_current_page(GTK_NOTEBOOK(w->nb));
 	gtk_clist_select_row(clist, num, FnumCol);
 	g_snprintf(buf, 4, "%d", num + 1);
 	gtk_clist_set_text(clist, num, FnumCol, buf);
@@ -606,9 +707,9 @@ void
 doc_list_show(GtkWidget *wgt, gpointer cbdata)
 {
 	int num;
-	GList *dp;
+	GSList *dp;
 	doc_t *d;
-	GtkWidget *tmp, *vbox;
+	GtkWidget *tmp, *vbox, *scrolled_win;
 
 	char *titles[] = { " # ", " Size (bytes) ", " File name " };
 	win_t *w = (win_t *)cbdata;
@@ -624,7 +725,7 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	w->dlw = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(w->dlw), "Currently Open Files");
 	gtk_signal_connect(GTK_OBJECT(w->dlw), "destroy",
-		GTK_SIGNAL_FUNC(doc_list_destroy), cbdata);
+			   GTK_SIGNAL_FUNC(doc_list_destroy), cbdata);
 
 	vbox = gtk_vbox_new(FALSE, 10);
 	gtk_container_add(GTK_CONTAINER(w->dlw), vbox);
@@ -634,14 +735,23 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	w->dlw_data = gtk_clist_new_with_titles(FLW_NUM_COLUMNS, titles);
 	gtk_widget_show(w->dlw_data);
 	tmp = w->dlw_data;
+	gtk_clist_column_titles_passive(GTK_CLIST(tmp));
+#ifdef GTK_HAVE_FEATURES_1_1_0
+	gtk_clist_set_column_auto_resize(GTK_CLIST(tmp), FnumCol, TRUE);
+	gtk_clist_set_column_auto_resize(GTK_CLIST(tmp), FsizeCol, TRUE);
+	gtk_clist_set_column_auto_resize(GTK_CLIST(tmp), FnameCol, TRUE);
+	gtk_clist_set_column_min_width(GTK_CLIST(tmp), FnameCol, 64);
+#else
 	gtk_clist_set_column_width(GTK_CLIST(tmp), FnumCol, 25);
-	gtk_clist_column_title_passive(GTK_CLIST(tmp), FnumCol);
 	gtk_clist_set_column_width(GTK_CLIST(tmp), FsizeCol, 90);
-	gtk_clist_column_title_passive(GTK_CLIST(tmp), FsizeCol);
-	gtk_clist_column_title_passive(GTK_CLIST(tmp), FnameCol);
-	gtk_clist_set_policy(
-		GTK_CLIST(tmp), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 0);
+#endif
+	scrolled_win = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scrolled_win), tmp);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
+	gtk_widget_show(scrolled_win);
 
 	num = 0;
 	dp = w->doclist;
@@ -657,14 +767,13 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	 * we keep changing the cur doc to the newly added file.
 	 */
 	gtk_signal_connect(GTK_OBJECT(w->dlw_data),
-		"select_row", GTK_SIGNAL_FUNC(doc_list_select), w);
+			   "select_row", GTK_SIGNAL_FUNC(doc_list_select), w);
 
 	tmp = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, TRUE, 0);
 	gtk_widget_show(tmp);
 
-	toolbar_create_dlw(w);
-	gtk_box_pack_start(GTK_BOX(vbox), w->dlw_toolbar_h, FALSE, FALSE, 0);
+	toolbar_create(w, vbox, dlw_tbdata, &(w->dlw_tb), &(w->dlw_tb_h));
 
 	/*
 	 * there should be a way to auto-detect how wide to make these widgets
@@ -677,7 +786,7 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	 * notebook, the notebook list and the list here are in reverse
 	 * order.  hence, manually select the actual current file.
 	 */
-	num = g_list_index(w->doclist, doc_current(w));
+	num = g_slist_index(w->doclist, doc_current(w));
 	gtk_clist_select_row(GTK_CLIST(w->dlw_data), num, FnumCol);
 
 	gtk_widget_show(w->dlw);
@@ -690,12 +799,13 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
  * creates a new document
  */
 void
-doc_new(win_t *w, char *fname)
+doc_new(win_t *w, char *fname, bool_t update_msgbar)
 {
+#ifdef APP_GNP
 	GtkText *text;
+#endif
 	char *cwd, *fn;
 	doc_t *d;
-
 	d = (doc_t *)g_malloc0(sizeof(doc_t));
 	d->docid = nextdocid++;
 	d->w = w;
@@ -703,11 +813,12 @@ doc_new(win_t *w, char *fname)
 
 	w->numdoc++;
 
+#ifdef APP_GNP
 	/* text widget */
-	d->txt = gtk_text_new(NULL, NULL);
+	d->data = gtk_text_new(NULL, NULL);
 
 	/* this is a hack to get consistent tab stops for the text widget */
-	text = GTK_TEXT(d->txt);
+	text = GTK_TEXT(d->data);
 	text->default_tab_width = 8;
 	text->tab_stops = g_list_remove(text->tab_stops, text->tab_stops->data);
 	text->tab_stops = g_list_remove(text->tab_stops, text->tab_stops->data);
@@ -715,13 +826,18 @@ doc_new(win_t *w, char *fname)
 	text->tab_stops = g_list_prepend(text->tab_stops, (void *)8);
 	text->tab_stops = g_list_prepend(text->tab_stops, (void *)8);
 
-	gtk_text_set_editable(GTK_TEXT(d->txt), TRUE);
-	gtk_text_set_point(GTK_TEXT(d->txt), 0);
-	gtk_text_set_word_wrap(GTK_TEXT(d->txt), IS_USE_WORDWRAP());
-	gtk_widget_show(d->txt);
+	gtk_text_set_editable(GTK_TEXT(d->data), TRUE);
+	gtk_text_set_point(GTK_TEXT(d->data), 0);
+	gtk_text_set_word_wrap(GTK_TEXT(d->data), IS_USE_WORDWRAP());
+	gtk_widget_show(d->data);
 #ifdef USE_AUTO_INDENT
-	gtk_signal_connect_after(GTK_OBJECT(d->txt), "key_press_event",
-		GTK_SIGNAL_FUNC(doc_auto_indent_cb), w);
+	gtk_signal_connect_after(GTK_OBJECT(d->data), "key_press_event",
+				 GTK_SIGNAL_FUNC(doc_auto_indent_cb), w);
+#endif
+	/* quickmenu */
+	gtk_signal_connect_object(GTK_OBJECT(d->data), "event",
+				  GTK_SIGNAL_FUNC(quickmenu_show),
+				  GTK_OBJECT(w->quickmenu));
 #endif
 
 	/* tab label */
@@ -730,22 +846,30 @@ doc_new(win_t *w, char *fname)
 	if (IS_SHOW_TABS())
 		gtk_widget_show(d->tablabel);
 
-	/* quickmenu */
-	gtk_signal_connect_object(GTK_OBJECT(d->txt), "event",
-		GTK_SIGNAL_FUNC(quickmenu_show), GTK_OBJECT(w->quickmenu));
-
 	/* add to notebook, update doclist, and update winlist popup */
 	doc_add_to_notebook(d);
-	w->doclist = g_list_insert(w->doclist, d, 0);
+	w->doclist = g_slist_prepend(w->doclist, d);
 	w->curdoc = d;
 	win_list_set_curdoc(w);
 	win_list_set_numdoc(w);
 
+#ifdef APP_GNP
 	/* now open the file and stuff the text into the text widget */
-	gtk_text_freeze(GTK_TEXT(d->txt));
-	gtk_widget_realize(d->txt);
+	gtk_text_freeze(GTK_TEXT(d->data));
+	gtk_widget_realize(d->data);
 	file_open_execute(w, d);
-	gtk_text_thaw(GTK_TEXT(d->txt));
+	doc_redraw(d);	/* hack alert!  it seems that even if the text
+			 * properties (font, fg and bg colors) are updated, we
+			 * can't use them in gtk_text_insert because the text
+			 * widget itself already has the settings which were
+			 * loaded at app startup.  so by putting the redraw
+			 * here, we force it to load whatever the current
+			 * settings are, as opposed to what was found during
+			 * app startup.  sometimes, the way GTK does is just
+			 * totally non-intuitive (chalk another one up to lack
+			 * of documentation). */
+	gtk_text_thaw(GTK_TEXT(d->data));
+#endif
 
 	/* various status updates */
 	win_set_title(d);
@@ -754,18 +878,21 @@ doc_new(win_t *w, char *fname)
 	/* skip printing cwd in full pathname */
 	fn = d->fname;
 	if ((cwd = getcwd(NULL, 0)) != NULL) {
-		if (strstr(d->fname, cwd)) {
+		if (strstr(d->fname, cwd))
 			fn = d->fname + strlen(cwd) + 1;
-		}
 		g_free(cwd);
 	}
 
+#ifdef APP_GNP
 	/* changed */
 	d->changed = FALSE;
-	d->changed_id = gtk_signal_connect(GTK_OBJECT(d->txt), "changed",
-		GTK_SIGNAL_FUNC(doc_changed_cb), d);
+	d->changed_id = gtk_signal_connect(GTK_OBJECT(d->data), "changed",
+					   GTK_SIGNAL_FUNC(doc_changed_cb), d);
+#endif
 
-	msgbar_printf(w, "Opened %s", fn);
+	doc_info_label_update(w);
+	if (update_msgbar)
+		msgbar_printf(w, "Opened %s", fn);
 	msgbox_printf("opened %s", fn);
 } /* doc_new */
 
@@ -778,7 +905,7 @@ doc_new(win_t *w, char *fname)
 void
 doc_new_cb(GtkWidget *wgt, gpointer cbdata)
 {
-	doc_new((win_t *)cbdata, UNTITLED);
+	doc_new((win_t *)cbdata, UNTITLED, TRUE);
 } /* doc_new_cb */
 
 
@@ -805,7 +932,7 @@ doc_open_cb(GtkWidget *wgt, gpointer cbdata)
 } /* doc_open_cb */
 
 
-#ifdef WANT_PROJECT
+#if defined(WANT_PROJECT) && defined(APP_GNP)
 void
 doc_open_prj(win_t *w)
 {
@@ -859,7 +986,9 @@ doc_open_in_new_win(GtkWidget *wgt, gpointer cbdata)
 
 	g_assert(w != NULL);
 	d = doc_current(w);
+#ifdef APP_GNP
 	if (!d->changed && strcmp(d->fname, UNTITLED) == 0)
+#endif
 		return;
 
 	fname = g_strdup(d->fname);
@@ -868,11 +997,11 @@ doc_open_in_new_win(GtkWidget *wgt, gpointer cbdata)
 		return;
 	}
 	if (w->numdoc < 1)
-		doc_new(w, UNTITLED);
+		doc_new(w, UNTITLED, TRUE);
 	recent_list_write(w);
 
 	neww = win_new();
-	doc_new(neww, fname);
+	doc_new(neww, fname, TRUE);
 	recent_list_add(neww, fname);
 	g_free(fname);
 
@@ -886,6 +1015,23 @@ doc_open_in_new_win(GtkWidget *wgt, gpointer cbdata)
 } /* doc_open_in_new_win */
 
 
+#ifdef APP_GNP
+/*
+ * PUBLIC: doc_redraw
+ *
+ * redraws the widgets for a text document
+ */
+void
+doc_redraw(void *data)
+{
+	doc_t *dp = (doc_t *)data;
+
+	prefs_update_text_widget_style(dp->data);
+} /* doc_redraw */
+#endif
+
+
+#ifdef APP_GNP
 /*
  * PUBLIC: doc_save_cb
  *
@@ -917,11 +1063,12 @@ doc_save_common(win_t *w)
 	if (strcmp(d->fname, UNTITLED) == 0)
 		ret = doc_saveas_common(w);
 	else
-		ret = file_save_execute(d);
+		ret = file_save_execute(d, FALSE);
 
 	if (ret) {
-		num = g_list_index(w->doclist, doc_current(w));
+		num = g_slist_index(w->doclist, doc_current(w));
 		doc_list_update(w, d, num, d->fname); 
+		doc_info_label_update(w);
 	}
 
 	recent_list_write(w);
@@ -968,11 +1115,11 @@ doc_saveas_common(win_t *w)
 	w->saveas = gtk_file_selection_new("Save As...");
 	tmp = w->saveas;
 	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(tmp)->ok_button),
-		"clicked", (GtkSignalFunc)doc_saveas_ok, w);
+			   "clicked", (GtkSignalFunc)doc_saveas_ok, w);
 	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(tmp)->cancel_button),
-		"clicked", (GtkSignalFunc)doc_saveas_destroy, w);
+			   "clicked", (GtkSignalFunc)doc_saveas_destroy, w);
 	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(tmp)->cancel_button),
-		"destroy", (GtkSignalFunc)doc_saveas_destroy, w);
+			   "destroy", (GtkSignalFunc)doc_saveas_destroy, w);
 
 	gtk_widget_show(w->saveas);
 
@@ -981,6 +1128,7 @@ doc_saveas_common(win_t *w)
 
 	return (d->changed_id != last_id);
 } /* doc_saveas_common */
+#endif
 
 
 /*** local function definitions ***/
@@ -992,7 +1140,10 @@ doc_saveas_common(win_t *w)
 static void
 doc_add_to_notebook(doc_t *d)
 {
-	GtkWidget *table, *vsb, *vbox1, *vbox2;
+#ifdef APP_GNP
+	GtkWidget *vsb, *vbox2;
+#endif
+	GtkWidget *table, *vbox1;
 	GtkStyle *style;
 
 	vbox1 = gtk_vbox_new(TRUE, TRUE);
@@ -1004,22 +1155,30 @@ doc_add_to_notebook(doc_t *d)
 	gtk_box_pack_start(GTK_BOX(vbox1), table, TRUE, TRUE, 1);
 	gtk_widget_show(table);
 
-	gtk_table_attach_defaults(GTK_TABLE(table), d->txt, 0, 1, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(table), d->data, 0, 1, 0, 1);
 
 	style = gtk_style_new();
-	gtk_widget_set_style(GTK_WIDGET(d->txt), style);
-	gtk_widget_set_rc_style(GTK_WIDGET(d->txt));
-	gtk_widget_ensure_style(GTK_WIDGET(d->txt));
+	gtk_widget_set_style(GTK_WIDGET(d->data), style);
+	gtk_widget_set_rc_style(GTK_WIDGET(d->data));
+	gtk_widget_ensure_style(GTK_WIDGET(d->data));
 
+#ifdef APP_GNP
 	vbox2 = gtk_vbox_new(TRUE, TRUE);
 	gtk_widget_show(vbox2);
 
-	vsb = gtk_vscrollbar_new(GTK_TEXT(d->txt)->vadj);
+	vsb = gtk_vscrollbar_new(GTK_TEXT(d->data)->vadj);
 	gtk_box_pack_start(GTK_BOX(vbox2), vsb, TRUE, TRUE, 0);
 	gtk_table_attach(GTK_TABLE(table), vbox2, 1, 2, 0, 1,
-		GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+			 (GtkAttachOptions)GTK_FILL,
+			 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+#else
+			 GTK_FILL, GTK_EXPAND | GTK_FILL,
+#endif
+			 0, 0);
 	GTK_WIDGET_UNSET_FLAGS(vsb, GTK_CAN_FOCUS);
 	gtk_widget_show(vsb);
+#endif
 
 	/*
 	 * if there are a lot of files open, appending is really slow since
@@ -1032,12 +1191,11 @@ doc_add_to_notebook(doc_t *d)
 	 * gtk_notebook_append_page(GTK_NOTEBOOK(d->w->notebook), vbox1,
 	 * 	d->tablabel);
 	 */
-	gtk_notebook_insert_page(GTK_NOTEBOOK(d->w->notebook), vbox1,
-		d->tablabel, 0);
+	gtk_notebook_insert_page(GTK_NOTEBOOK(d->w->nb), vbox1, d->tablabel, 0);
 
-	gtk_notebook_set_page(GTK_NOTEBOOK(d->w->notebook), 0);
+	gtk_notebook_set_page(GTK_NOTEBOOK(d->w->nb), 0);
 
-	gtk_widget_grab_focus(d->txt);
+	gtk_widget_grab_focus(d->data);
 } /* doc_add_to_notebook */
 
 
@@ -1077,6 +1235,7 @@ doc_auto_indent_cb(GtkWidget *txt, GdkEventKey *event, win_t *w)
 		buffer = gtk_editable_get_chars(GTK_EDITABLE(txt), i - 1, i);
 		if (buffer == NULL)
 			continue;
+
 		if (buffer[0] == 10) {
 			if (newlines > 0) {
 				g_free(buffer);
@@ -1102,7 +1261,7 @@ doc_auto_indent_cb(GtkWidget *txt, GdkEventKey *event, win_t *w)
 	if (strlen(whitespace) > 0) {
 		i = GTK_EDITABLE(txt)->current_pos;
 		gtk_editable_insert_text(GTK_EDITABLE(txt), whitespace,
-			strlen(whitespace), &i);
+					 strlen(whitespace), &i);
 	}
 	g_free(whitespace);
 
@@ -1124,8 +1283,10 @@ doc_close_common(win_t *w)
 	g_assert(w != NULL);
 	d = doc_current(w);
 
+#ifdef APP_GNP
 	if (d->changed)
 		return doc_close_verify(w);
+#endif
 
 	return doc_close_execute(w);
 } /* doc_close_common */
@@ -1150,6 +1311,7 @@ doc_close_execute(win_t *w)
 } /* doc_close_execute */
 
 
+#ifdef APP_GNP
 /*
  * PRIVATE: doc_close_verify
  *
@@ -1207,20 +1369,21 @@ doc_close_verify_yes(win_t *w)
 		if (d->changed == FALSE)
 			return doc_close_execute(w);
 	} else {
-		saved = file_save_execute(d);
+		saved = file_save_execute(d, FALSE);
 		if (saved)
 			return doc_close_execute(w);
 	}
 
 	return FALSE;
 } /* doc_close_verify_yes */
+#endif
 
 
 /*
  * PRIVATE: doc_free
  *
  * free space used by a doc_t.  currently, only used after doc_remove(),
- * so there is some g_list magic happening.
+ * so there is some g_slist magic happening.
  */
 static void
 doc_free(doc_t *d)
@@ -1246,8 +1409,8 @@ doc_free(doc_t *d)
 	if (d->printentry)
 		gtk_widget_destroy(d->printentry);
 #if GLIST_MAGIC
-	if (d->txt)
-		gtk_widget_destroy(d->txt);
+	if (d->data)
+		gtk_widget_destroy(d->data);
 	if (d->tablabel)
 		gtk_widget_destroy(d->tablabel);
 	g_free(d);
@@ -1265,7 +1428,7 @@ static void
 doc_info_advanced(GtkWidget *wgt, gpointer cbdata)
 {
 	doc_info_t *di = (doc_info_t *)cbdata;
-	GList *alp;
+	GSList *alp;
 	GtkWidget *wp;
 
 	g_assert(di != NULL);
@@ -1278,7 +1441,7 @@ doc_info_advanced(GtkWidget *wgt, gpointer cbdata)
 
 	gtk_widget_destroy(di->adv_button);
 	gtk_window_set_title(GTK_WINDOW(di->toplev),
-			"Advanced File Information");
+			     "Advanced File Information");
 } /* doc_info_advanced */
 
 
@@ -1293,8 +1456,9 @@ doc_info_close(GtkWidget *wgt, gpointer cbdata)
 	doc_t *d = (doc_t *)cbdata;
 	doc_info_t *di;
 
+	g_assert(d != NULL);
 	di = (doc_info_t *)(d->docinfo);
-	gtk_widget_destroy(di->toplev);
+	gtk_widget_destroy(di->toplev);	/* goes to doc_info_destroy */
 } /* doc_info_close */
 
 
@@ -1308,7 +1472,7 @@ doc_info_destroy(GtkWidget *wgt, gpointer cbdata)
 {
 	doc_t *d = (doc_t *)cbdata;
 	doc_info_t *di;
-	GList *alp;
+	GSList *alp;
 	GtkWidget *wp;
 
 	g_assert(d != NULL);
@@ -1319,7 +1483,7 @@ doc_info_destroy(GtkWidget *wgt, gpointer cbdata)
 		gtk_widget_destroy(wp);
 		alp = alp->next;
 	}
-	g_list_free(di->adv_list);
+	g_slist_free(di->adv_list);
 	g_free(di);
 	d->docinfo = NULL;
 } /* doc_info_destroy */
@@ -1343,7 +1507,7 @@ doc_info_table_add(GtkWidget *table, doc_info_t *di, char *text,
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, lft, rht, top, bot);
 
 	if (advanced)
-		di->adv_list = g_list_append(di->adv_list, tmp);
+		di->adv_list = g_slist_prepend(di->adv_list, tmp);
 	else
 		gtk_widget_show(tmp);
 } /* doc_info_table_add */
@@ -1356,7 +1520,7 @@ doc_info_table_add(GtkWidget *table, doc_info_t *di, char *text,
  * effectively, sets the notebook to show the selected file.
  * see select_row() in gtkclist.c to match function parameters.
  */
-static void
+static int
 doc_list_select(GtkWidget *wgt, int row, int column,
 	GdkEventButton *event, gpointer cbdata)
 {
@@ -1368,7 +1532,9 @@ doc_list_select(GtkWidget *wgt, int row, int column,
 	 * pointer in the win_t field at this point since it's down
 	 * in our switch_page callback (see notebook_switch_page()).
 	 */
-	gtk_notebook_set_page(GTK_NOTEBOOK(w->notebook), row);
+	gtk_notebook_set_page(GTK_NOTEBOOK(w->nb), row);
+
+	return TRUE;
 } /* doc_list_select */
 
 
@@ -1460,12 +1626,15 @@ doc_open_ok(GtkWidget *wgt, gpointer cbdata)
 		 * doc, then remove it before adding the one we want
 		 */
 		d = doc_current(w);
-		if (w->numdoc == 1 && !d->changed &&
-			strcmp(d->fname, UNTITLED) == 0) {
+		if (w->numdoc == 1 &&
+#ifdef APP_GNP
+		    !d->changed &&
+#endif
+		    strcmp(d->fname, UNTITLED) == 0) {
 			doc_close_common(w);
 		}
 
-		doc_new(w, fname);
+		doc_new(w, fname, TRUE);
 		recent_list_add(w, fname);
 	} /* fname != NULL */
 
@@ -1483,14 +1652,14 @@ out:
 static doc_t *
 doc_remove(win_t *w)
 {
-	GList *dlp;
+	GSList *dlp;
 	GtkNotebook *nb;
 	char *cwd, *fn;
 	doc_t *d;
 	int num;
 
 	d = doc_current(w);
-	nb = GTK_NOTEBOOK(w->notebook);
+	nb = GTK_NOTEBOOK(w->nb);
 	g_assert(nb != NULL);
 
 	/* remove from notebook */
@@ -1501,15 +1670,15 @@ doc_remove(win_t *w)
 	 * remove from doclist and doc list window.  update numdoc in winlist.
 	 * contents of the doc gets freed in doc_free().
 	 */
-	dlp = g_list_find(w->doclist, d);
-	w->doclist = g_list_remove_link(w->doclist, dlp);
+	dlp = g_slist_find(w->doclist, d);
+	w->doclist = g_slist_remove_link(w->doclist, dlp);
 	w->numdoc--;
 	doc_list_remove(w, num);
 	win_list_set_numdoc(w);
 
 	/* sanity checks */
 	g_assert(w->numdoc == g_list_length(nb->children));
-	g_assert(w->numdoc == g_list_length(w->doclist));
+	g_assert(w->numdoc == g_slist_length(w->doclist));
 
 	/* skip printing cwd in full pathname */
 	fn = d->fname;
@@ -1529,13 +1698,14 @@ doc_remove(win_t *w)
 		return (doc_t *)(dlp->data);
 	}
 
-	w->curdoc = g_list_nth_data(w->doclist, num);
-	win_set_title((doc_t *)(w->curdoc));
+	w->curdoc = g_slist_nth_data(w->doclist, num);
+	win_set_title(doc_current(w));
 
 	return (doc_t *)(dlp->data);
 } /* doc_remove */
 
 
+#ifdef APP_GNP
 /*
  * PRIVATE: doc_saveas_destroy
  *
@@ -1549,8 +1719,10 @@ doc_saveas_destroy(GtkWidget *wgt, gpointer cbdata)
 	gtk_widget_destroy(w->saveas);
 	w->saveas = NULL;
 } /* doc_saveas_destroy */
+#endif
 
 
+#ifdef APP_GNP
 /*
  * PRIVATE: doc_saveas_ok
  *
@@ -1607,13 +1779,15 @@ saveas_do_save:
 		if (d->fname)
 			g_free(d->fname);
 		d->fname = g_strdup(fname);
-		saved = file_save_execute(d);
+		saved = file_save_execute(d, FALSE);
 		if (saved) {
 			gtk_widget_destroy(w->saveas);
 			w->saveas = NULL;
+			doc_info_label_update(w);
 		}
 	}
 } /* doc_saveas_ok */
+#endif
 
 
 /* the end */
