@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: misc.h,v 1.3 1998/10/12 14:49:57 kahn Exp kahn $
+ * $Id: misc.h,v 1.5 1999/01/18 20:03:25 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -25,12 +25,26 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <time.h>
+#include <gtk/gtk.h>
 #include "main.h"
 
+extern char *my_tempname(void);
 extern char *ttoa(time_t curtime, bool_t append_nl);
 extern char *ltoa(long num);
 extern char * strcasestr(const char *phaystack, const char *pneedle);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+#define my_basename(name)	g_basename((const gchar *)(name))
+#else
 extern const char *my_basename(const char *name);
+#endif
+
+extern GtkWidget *misc_button_new_w_label(
+			char *labelname, GtkSignalFunc cbfunc,
+			gpointer cbdata, GtkWidget *parent,
+#ifdef GTK_HAVE_FEATURES_1_1_0
+			GtkReliefStyle relief,
+#endif
+			GtkWidget *table, int lft, int rht, int top, int bot);
 
 
 #ifdef __cplusplus
