@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: msgbar.h,v 1.5 1998/10/05 15:33:07 kahn Exp kahn $
+ * $Id: msgbar.h,v 1.7 1999/01/06 20:55:58 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -29,10 +29,10 @@ extern "C" {
 #include <gtk/gtk.h>
 #include "win.h"
 
-extern void	msgbar_clear(gpointer data);
+extern int	msgbar_clear(gpointer data);
 extern void	msgbar_init(win_t *w, GtkWidget *parent);
 extern void	msgbar_printf(win_t *w, const char *fmt, ...);
-extern void	msgbar_redraw(win_t *w);
+extern void	msgbar_redraw(void *);
 extern void	msgbar_toggle(GtkWidget *wgt, gpointer cbdata);
 extern void	msgbar_timeout_add(win_t *w);
 
