@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: prjbar.h,v 1.4 1998/10/05 15:34:19 kahn Exp kahn $
+ * $Id: prjbar.h,v 1.6 1999/01/18 20:06:35 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -24,16 +24,17 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#ifdef WANT_PROJECT
 
 #include <stdarg.h>
 #include <gtk/gtk.h>
+#include "main.h"
+#ifdef WANT_PROJECT
 #include "win.h"
 
 extern void	prjbar_clear(gpointer data);
 extern void	prjbar_init(win_t *w);
 extern void	prjbar_update(win_t *w);
-extern void	prjbar_redraw(win_t *w);
+extern void	prjbar_redraw(void *);
 extern void	prjbar_toggle(GtkWidget *wgt, gpointer cbdata);
 
 #endif	/* WANT_PROJECT */
