@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: win.c,v 1.15 1998/10/16 20:21:01 kahn Exp kahn $
+ * $Id: win.c,v 1.30 1999/01/21 21:38:18 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
  *
@@ -28,8 +28,13 @@
 #include "msgbox.h"
 #include "prefs.h"
 #include "recent.h"
+#ifdef APP_GNP
 #include "prjbar.h"
-
+#include "print.h"
+#include "clipboard.h"
+#include "search.h"
+#include "htmltags.h"
+#endif
 
 /*** external declarations ***/
 extern void	win_set_title(doc_t *d);
@@ -43,31 +48,64 @@ extern void	win_set_title(doc_t *d);
 #define DOC_TABS_BOTTOM	"Document tabs on the BOTTOM"
 #define DOC_TABS_LEFT	"Document tabs on the LEFT"
 #define DOC_TABS_RIGHT	"Document tabs on the RIGHT"
+#define DOC_WWRAP_OFF	"Wordwrap OFF"
+#define DOC_WWRAP_ON	"Wordwrap ON"
 
 
 /*** local variables ***/
-static GList *winlist = NULL;		/* list of win_t's */
+static GSList *winlist = NULL;		/* list of win_t's */
 static unsigned nextid = 10000;		/* internal window id for each win_t */
 
 static GtkWidget *wlw = NULL;		/* these three are used for the */
 static GtkWidget *wlw_data = NULL;	/* winlist popup window */
 static unsigned curwinrow = 0;		/* current window rownum */
 
+static toolbar_data_t main_tbdata[] = {
+	{ " New ",	"New File",	"new",		"tb_new.xpm",
+			(GtkSignalFunc)doc_new_cb },
+	{ " Open ",	"Open File",	"open",		"tb_open.xpm",
+			(GtkSignalFunc)doc_open_cb },
+	{ " Close ",	"Close File",	"close",	"tb_cancel.xpm",
+			(GtkSignalFunc)doc_close_cb },
+#ifdef APP_GNP
+	{ " Save ",	"Save File",	"save",		"tb_save.xpm",
+			(GtkSignalFunc)doc_save_cb },
+	{ " Print ",	"Print File",	"print",	"tb_print.xpm",
+			(GtkSignalFunc)print_cb },
+	{ " SPACE " },
+	{ " Cut ",	"Cut Text",	"cut",		"tb_cut.xpm",
+			(GtkSignalFunc)cut_cb },
+	{ " Copy ",	"Copy Text",	"copy",		"tb_copy.xpm",
+			(GtkSignalFunc)copy_cb },
+	{ " Paste ",	"Paste Text",	"paste",	"tb_paste.xpm",
+			(GtkSignalFunc)paste_cb	},
+	{ " Find ",	"Find Text",	"Find",		"tb_search.xpm",
+			(GtkSignalFunc)search_cb },
+#endif
+	{ " SPACE " },
+	{ " Prefs ",	"Preferences",	"prefs",	"tb_prefs.xpm",
+			(GtkSignalFunc)prefs_cb },
+	{ " Exit ",	"Exit App",	"exit",		"tb_exit.xpm",
+			(GtkSignalFunc)win_close_all_cb },
+	{ NULL }
+};
+
 
 /*** local function prototypes ***/
-static void notebook_init(win_t *w);
-static void notebook_switch_page(GtkWidget *wgt, GtkNotebookPage *page,
-	int num, gpointer cbdata);
-static bool_t win_close_common(win_t *w);
-static void win_list_destroy(GtkWidget *wgt, gpointer cbdata);
-static void win_list_remove(win_t *w);
-static void win_list_add(win_t *w, char *title);
-static void win_list_select(GtkWidget *wgt, unsigned row, int column,
-	GdkEventButton *event, gpointer cbdata);
-static void win_list_close_selected(GtkWidget *wgt, gpointer cbdata);
+static void	notebook_init(win_t *w);
+static void	notebook_switch_page(GtkWidget *wgt, GtkNotebookPage *page,
+				     int num, gpointer cbdata);
+static bool_t	win_close_common(win_t *w);
+static void	win_list_destroy(GtkWidget *wgt, gpointer cbdata);
+static void	win_list_remove(win_t *w);
+static void	win_list_add(win_t *w, char *title);
+static int	win_list_select(GtkWidget *wgt, unsigned row, int column,
+				GdkEventButton *event, gpointer cbdata);
+static void	win_list_close_selected(GtkWidget *wgt, gpointer cbdata);
 
 
 /*** global function definitions ***/
+#ifdef APP_GNP
 /*
  * PUBLIC: win_autosave
  *
@@ -77,7 +115,7 @@ static void win_list_close_selected(GtkWidget *wgt, gpointer cbdata);
 int
 win_autosave(gpointer cbdata)
 {
-	GList *wlp, *dlp;
+	GSList *wlp, *dlp;
 	win_t *w;
 	doc_t *d;
 	int num = 0;
@@ -87,11 +125,11 @@ win_autosave(gpointer cbdata)
 	while (wlp) {
 		w = (win_t *)(wlp->data);
 		msgbar_printf(w, AUTOSAVE_MSG);
-		dlp = (GList *)(w->doclist);
+		dlp = (GSList *)(w->doclist);
 		while (dlp) {
 			d = (doc_t *)(dlp->data);
-			if (d->changed) {
-				(void)file_save_execute(d);
+			if (d->changed && strcmp(d->fname, UNTITLED)) {
+				(void)file_save_execute(d, FALSE);
 				num++;
 			}
 			dlp = dlp->next;
@@ -102,98 +140,45 @@ win_autosave(gpointer cbdata)
 
 	return 1;	/* for GTK, must return 1 */
 } /* win_new_with_new_doc */
+#endif
 
 
 /*
- * PUBLIC: win_redraw_doc_tab
+ * PUBLIC: win_foreach
  *
- * for all windows, redraw the document tabs.  called from prefs_save, when
- * the preferences may have changed.
+ * traver window list and call a function on each window.
  */
 void
-win_redraw_doc_tab(gpointer data)
+win_foreach(void (*func)(void *))
 {
-	GList *wlp;
+	GSList *wlp;
 	win_t *wp;
 
 	wlp = winlist;
 	while (wlp) {
 		wp = (win_t *)(wlp->data);
-		if (IS_SHOW_TABS())
-			gtk_notebook_set_show_tabs(
-					GTK_NOTEBOOK(wp->notebook), TRUE);
-		else
-			gtk_notebook_set_show_tabs(
-					GTK_NOTEBOOK(wp->notebook), FALSE);
-		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(wp->notebook), tabpos);
+		func(wp);
 		wlp = wlp->next;
 	}
-} /* win_redraw_doc_tab */
+} /* win_foreach */
 
 
 /*
- * PUBLIC: win_redraw_msgbar
+ * PUBLIC: win_doc_foreach
  *
- * for all windows, redraw the toolbar.  called from prefs_save, when
- * the preferences may have changed.
+ * traverse window list, and traverse all docs in each window.
  */
 void
-win_redraw_msgbar(gpointer data)
+win_doc_foreach(void (*func)(void *))
 {
-	GList *wlp;
+	GSList *wlp;
 	win_t *wp;
 
-	wlp = winlist;
-	while (wlp) {
+	for (wlp = winlist; wlp; wlp = wlp->next) {
 		wp = (win_t *)(wlp->data);
-		msgbar_redraw(wp);
-		wlp = wlp->next;
+		doc_foreach(wp, func);
 	}
-} /* win_redraw_msgbar */
-
-
-#ifdef WANT_PROJECT
-/*
- * PUBLIC: win_redraw_prjbar
- *
- * for all windows, redraw the toolbar.  called from prefs_save, when
- * the preferences may have changed.
- */
-void
-win_redraw_prjbar(gpointer data)
-{
-	GList *wlp;
-	win_t *wp;
-
-	wlp = winlist;
-	while (wlp) {
-		wp = (win_t *)(wlp->data);
-		prjbar_redraw(wp);
-		wlp = wlp->next;
-	}
-} /* win_redraw_prjbar */
-#endif	/* WANT_PROJECT */
-
-
-/*
- * PUBLIC: win_redraw_toolbar
- *
- * for all windows, redraw the toolbar.  called from prefs_save, when
- * the preferences may have changed.
- */
-void
-win_redraw_toolbar(gpointer data)
-{
-	GList *wlp;
-	win_t *wp;
-
-	wlp = winlist;
-	while (wlp) {
-		wp = (win_t *)(wlp->data);
-		tb_redraw(wp);
-		wlp = wlp->next;
-	}
-} /* win_redraw_toolbar */
+} /* win_doc_foreach */
 
 
 /*
@@ -208,7 +193,7 @@ win_new_with_doc(GtkWidget *wgt, gpointer cbdata)
 	win_t *w;
 
 	w = win_new();
-	doc_new(w, UNTITLED);
+	doc_new(w, UNTITLED, TRUE);
 	gtk_widget_show(w->toplev);
 } /* win_new_with_new_doc */
 
@@ -228,18 +213,18 @@ win_new(void)
 
 	/* toplevel */
 	w->toplev = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_widget_set_name(w->toplev, "eedit");
-	gtk_window_set_wmclass(GTK_WINDOW(w->toplev), "eedit", "eedit");
-	gtk_window_set_title(GTK_WINDOW(w->toplev), "Title");
+	gtk_widget_set_name(w->toplev, "gnp+");
+	gtk_window_set_wmclass(GTK_WINDOW(w->toplev), "gnp+", "gnp+");
+	gtk_window_set_title(GTK_WINDOW(w->toplev), "gnotepad+");
 	gtk_window_set_policy(GTK_WINDOW(w->toplev), TRUE, TRUE, FALSE);
 	gtk_container_border_width(GTK_CONTAINER(w->toplev), 0);
 	gtk_signal_connect(GTK_OBJECT(w->toplev), "destroy",
-		GTK_SIGNAL_FUNC(win_close_cb), w);
+			   GTK_SIGNAL_FUNC(win_close_cb), w);
 
-	gtk_widget_set_usize(GTK_WIDGET(w->toplev), win_width, win_height);
+	gtk_widget_set_usize(GTK_WIDGET(w->toplev), prefs.win_width, prefs.win_height);
 	if (IS_SAVE_WIN_POS())
 		gtk_widget_set_uposition(GTK_WIDGET(w->toplev),
-					win_xpos, win_ypos);
+					 prefs.win_xpos, prefs.win_ypos);
 
 	/* mainbox */
 	w->mainbox = gtk_vbox_new(FALSE, 0);
@@ -252,7 +237,13 @@ win_new(void)
 	/* these need to be in order */
 	menu_main_init(w);
 	recent_list_init(w);
-	toolbar_main_init(w);
+	gtk_widget_realize(w->toplev);
+	toolbar_create(w, w->mainbox, main_tbdata, &(w->main_tb),
+		       &(w->main_tb_h));
+#ifdef APP_GNP
+	html_tb_create(w, w->mainbox, &(w->html_tb), &(w->html_tb_h));
+#endif
+
 	notebook_init(w);
 #ifdef WANT_PROJECT
 	prjbar_init(w);
@@ -265,10 +256,12 @@ win_new(void)
 	msgbar_init(w, w->hbox_bot);
 	doc_info_init(w, w->hbox_bot);
 
+#ifdef APP_GNP
 	quickmenu_init(w);
+#endif
 
 	/* add to winlist and winlist popup */
-	winlist = g_list_append(winlist, (gpointer)w);
+	winlist = g_slist_append(winlist, (gpointer)w);
 	win_list_add(w, GTK_WINDOW(w->toplev)->title);
 
 	return w;
@@ -295,7 +288,41 @@ win_close_cb(GtkWidget *wgt, gpointer cbdata)
 void
 win_close_all_cb(GtkWidget *wgt, gpointer cbdata)
 {
+	GSList *wlp;
+	win_t *w;
+	bool_t changed = FALSE;
+
 	msgbox_close();
+	if (wlw)
+		gtk_widget_hide(wlw);
+
+	for (wlp = winlist; wlp; wlp = wlp->next) {
+		w = (win_t *)(wlp->data);
+#ifdef APP_GNP
+		if (doc_check_if_any_changed(w)) {
+			changed = TRUE;
+			break;
+		}
+#endif
+	}
+
+	if (!changed) {
+		for (wlp = winlist; wlp; wlp = wlp->next) {
+			w = (win_t *)(wlp->data);
+			gtk_widget_hide(w->toplev);
+#ifdef APP_GNP
+			if (w->saveas)
+				gtk_widget_hide(w->saveas);
+			if (w->search)
+				gtk_widget_hide(w->search);
+#endif
+			if (w->filesel)
+				gtk_widget_hide(w->filesel);
+			if (w->dlw)
+				gtk_widget_hide(w->dlw);
+		}
+	}
+
 	while (winlist) {
 		if (win_close_common((win_t *)(winlist->data)) == FALSE)
 			break;
@@ -316,11 +343,11 @@ win_dtab_toggle(GtkWidget *wgt, gpointer cbdata)
 	win_t *w = (win_t *)cbdata;
 
 	if (IS_SHOW_TABS()) {
-		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->notebook), FALSE);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->nb), FALSE);
 		CLEAR_SHOW_TABS();
 		msgbar_printf(w, DOC_TABS_OFF);
 	} else {
-		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->notebook), TRUE);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->nb), TRUE);
 		SET_SHOW_TABS();
 		msgbar_printf(w, DOC_TABS_ON);
 	}
@@ -337,8 +364,8 @@ win_dtab_top(GtkWidget *wgt, gpointer cbdata)
 {
 	win_t *w = (win_t *)cbdata;
 
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->notebook), GTK_POS_TOP);
-	tabpos = GTK_POS_TOP;
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), GTK_POS_TOP);
+	prefs.tabpos = GTK_POS_TOP;
 	msgbar_printf(w, DOC_TABS_TOP);
 } /* win_dtab_top */
 
@@ -353,8 +380,8 @@ win_dtab_bot(GtkWidget *wgt, gpointer cbdata)
 {
 	win_t *w = (win_t *)cbdata;
 
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->notebook), GTK_POS_BOTTOM);
-	tabpos = GTK_POS_BOTTOM;
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), GTK_POS_BOTTOM);
+	prefs.tabpos = GTK_POS_BOTTOM;
 	msgbar_printf(w, DOC_TABS_BOTTOM);
 } /* win_dtab_bot */
 
@@ -369,8 +396,8 @@ win_dtab_left(GtkWidget *wgt, gpointer cbdata)
 {
 	win_t *w = (win_t *)cbdata;
 
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->notebook), GTK_POS_LEFT);
-	tabpos = GTK_POS_LEFT;
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), GTK_POS_LEFT);
+	prefs.tabpos = GTK_POS_LEFT;
 	msgbar_printf(w, DOC_TABS_LEFT);
 } /* win_dtab_left */
 
@@ -385,12 +412,29 @@ win_dtab_right(GtkWidget *wgt, gpointer cbdata)
 {
 	win_t *w = (win_t *)cbdata;
 
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->notebook), GTK_POS_RIGHT);
-	tabpos = GTK_POS_RIGHT;
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), GTK_POS_RIGHT);
+	prefs.tabpos = GTK_POS_RIGHT;
 	msgbar_printf(w, DOC_TABS_RIGHT);
 } /* win_dtab_right */
 
 
+/*
+ * PUBLIC: win_redraw_doc_tab
+ *
+ * redraw the document tabs for a specified window.  called from prefs_save,
+ * when the preferences may have changed.
+ */
+void
+win_redraw_doc_tab(void *data)
+{
+	win_t *w = (win_t *)data;
+
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->nb), IS_SHOW_TABS());
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), prefs.tabpos);
+} /* win_redraw_doc_tab */
+
+
+#ifdef APP_GNP
 /*
  * PUBLIC: win_toggle_wwrap
  *
@@ -406,13 +450,16 @@ win_wwrap_toggle(GtkWidget *wgt, gpointer cbdata)
 	d = doc_current(w);
 	g_assert(d != NULL);
 	if (IS_USE_WORDWRAP()) {
-		gtk_text_set_word_wrap(GTK_TEXT(d->txt), FALSE);
+		gtk_text_set_word_wrap(GTK_TEXT(d->data), FALSE);
 		CLEAR_USE_WORDWRAP();
+		msgbar_printf(w, DOC_WWRAP_OFF);
 	} else {
-		gtk_text_set_word_wrap(GTK_TEXT(d->txt), TRUE);
+		gtk_text_set_word_wrap(GTK_TEXT(d->data), TRUE);
 		SET_USE_WORDWRAP();
+		msgbar_printf(w, DOC_WWRAP_ON);
 	}
 } /* win_wwrap_toggle */
+#endif	/* APP_GNP */
 
 
 /*** local function definitions ***/
@@ -425,79 +472,66 @@ win_wwrap_toggle(GtkWidget *wgt, gpointer cbdata)
 static bool_t
 win_close_common(win_t *w)
 {
-	g_assert(w != NULL);
+	doc_new_name_t *dnnp;
+	GSList *dlp, *wlp, *rlp;
 
 	doc_list_destroy(NULL, w);
 	win_list_destroy(NULL, NULL);
 
+	dlp = (GSList *)w->doclist;
+	if (dlp == NULL || w->numdoc < 1)
+		return TRUE;
+
+#ifdef APP_GNP
 	if (doc_check_if_any_changed(w) == FALSE)
 		gtk_widget_hide(w->toplev);
-
-	if (doc_close_all_common(w) == TRUE) {
-		doc_new_name_t *dnnp;
-		GList *wlp, *dlp, *rlp;
-		doc_t *d;
-
-		/* update preferences */
-		if (IS_SAVE_WIN_POS())
-			gdk_window_get_position(w->toplev->window,
-						&win_xpos, &win_ypos);
-
-		/* remove from the win list popup window */
-		win_list_remove(w);
-
-		/* remove from the linked list of win_t's */
-		wlp = g_list_find(winlist, w);
-		winlist = g_list_remove_link(winlist, wlp);
-
-		/* cleanup */
-		while (w->doclist) {
-			dlp = w->doclist;
-			w->doclist = g_list_remove_link(w->doclist, dlp);
-			d = (doc_t *)(dlp->data);
-			g_free(d->fname);
-			g_free(d->sb);
-			gtk_widget_destroy(d->txt);
-			gtk_widget_destroy(d->tablabel);
-			if (d->printwin)
-				gtk_widget_destroy(d->printwin);
-			if (d->printentry)
-				gtk_widget_destroy(d->printentry);
-			g_free(d);
-#if GLIST_MAGIC
-			g_free(dlp);
-#endif
-		}
-		while (w->recentlist) {
-			rlp = w->recentlist;
-			w->recentlist = g_list_remove_link(w->recentlist, rlp);
-			dnnp = (doc_new_name_t *)(rlp->data);
-			g_free(dnnp->fname);
-			g_free(dnnp);
-#if GLIST_MAGIC
-			g_free(rlp);
-#endif
-		}
-		g_free(w->lastmsg);
-		gtk_timeout_remove(w->timeout_id);
-		gtk_widget_destroy(w->toplev);
-		g_free(w);
-#if GLIST_MAGIC
-		g_free(wlp);
 #endif
 
-		/* save preferences */
-		if (IS_SAVE_WIN_WIDTH() || IS_SAVE_WIN_HEIGHT() ||
-				IS_SAVE_WIN_POS())
-			prefs_save();
+	if (doc_close_all_common(w) == FALSE)
+		return FALSE;
 
-		if (winlist == NULL)
-			gtk_exit(0);
+	dlp = (GSList *)w->doclist;
+	g_assert(dlp == NULL && w->numdoc == 0);
 
-		return TRUE;
+	/* update preferences */
+	if (IS_SAVE_WIN_POS())
+		gdk_window_get_position(w->toplev->window,
+					&prefs.win_xpos, &prefs.win_ypos);
+
+	/* remove from the win list popup window */
+	win_list_remove(w);
+
+	/* remove from the linked list of win_t's */
+	wlp = g_slist_find(winlist, w);
+	winlist = g_slist_remove_link(winlist, wlp);
+
+	/* cleanup */
+	while (w->recentlist) {
+		rlp = w->recentlist;
+		w->recentlist = g_slist_remove_link(w->recentlist, rlp);
+		dnnp = (doc_new_name_t *)(rlp->data);
+		g_free(dnnp->fname);
+		g_free(dnnp);
+#if GLIST_MAGIC
+		g_free(rlp);
+#endif
 	}
+	g_free(w->lastmsg);
+	gtk_timeout_remove(w->timeout_id);
+	gtk_widget_destroy(w->toplev);
+	g_free(w);
+#if GLIST_MAGIC
+	g_free(wlp);
+#endif
 
-	return FALSE;
+	/* save preferences */
+	if (IS_SAVE_WIN_WIDTH() || IS_SAVE_WIN_HEIGHT() || IS_SAVE_WIN_POS())
+		prefs_save();
+
+	if (winlist == NULL)
+		gtk_exit(0);
+
+	return TRUE;
 } /* win_close_common */
 
 
@@ -509,20 +543,21 @@ win_close_common(win_t *w)
 static void
 notebook_init(win_t *w)
 {
-	w->notebook = gtk_notebook_new();
-	gtk_notebook_set_scrollable(GTK_NOTEBOOK(w->notebook), TRUE);
-	gtk_signal_connect_after(GTK_OBJECT(w->notebook), "switch_page",
-		GTK_SIGNAL_FUNC(notebook_switch_page), w);
-	gtk_box_pack_start(GTK_BOX(w->mainbox), w->notebook, TRUE, TRUE, 0);
+	w->nb = gtk_notebook_new();
+	gtk_notebook_set_scrollable(GTK_NOTEBOOK(w->nb), TRUE);
+	gtk_notebook_popup_enable(GTK_NOTEBOOK(w->nb));
+	gtk_signal_connect_after(GTK_OBJECT(w->nb), "switch_page",
+				 GTK_SIGNAL_FUNC(notebook_switch_page), w);
+	gtk_box_pack_start(GTK_BOX(w->mainbox), w->nb, TRUE, TRUE, 0);
 
 	/* set according to preferences */
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->notebook), tabpos);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w->nb), prefs.tabpos);
 	if (IS_SHOW_TABS())
-		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->notebook), TRUE);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->nb), TRUE);
 	else
-		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->notebook), FALSE);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(w->nb), FALSE);
 
-	gtk_widget_show(w->notebook);
+	gtk_widget_show(w->nb);
 } /* notebook_init */
 
 
@@ -546,13 +581,15 @@ notebook_switch_page(GtkWidget *wgt, GtkNotebookPage *page,
 	if (!GTK_WIDGET_REALIZED(w->toplev))
 		return;
 
-	w->curdoc = g_list_nth_data(w->doclist, num);
+	w->curdoc = g_slist_nth_data(w->doclist, num);
 	if (w->dlw_data)
 		gtk_clist_select_row(GTK_CLIST(w->dlw_data), num, FnumCol);
-	d = (doc_t *)(w->curdoc);
-	gtk_widget_grab_focus(d->txt);
-	win_set_title(d);
-	win_list_set_curdoc(w);
+	if ((d = doc_current(w)) != NULL) {
+		gtk_widget_grab_focus(d->data);
+		win_set_title(d);
+		win_list_set_curdoc(w);
+		doc_info_label_update(w);
+	}
 } /* notebook_switch_page */
 
 
@@ -631,8 +668,8 @@ win_list_set_numdoc(win_t *w)
 void
 win_list_show(GtkWidget *wgt, gpointer cbdata)
 {
-	GtkWidget *tmp, *vbox, *hbox;
-	GList *wlp;
+	GtkWidget *tmp, *vbox, *hbox, *scrolled_win;
+	GSList *wlp;
 	win_t *wp;
 
 	char *titles[] = { " # ", " # Docs ", " Current Document " };
@@ -648,7 +685,7 @@ win_list_show(GtkWidget *wgt, gpointer cbdata)
 	wlw = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(wlw), "Window List");
 	gtk_signal_connect(GTK_OBJECT(wlw), "destroy",
-		GTK_SIGNAL_FUNC(win_list_destroy), NULL);
+			   GTK_SIGNAL_FUNC(win_list_destroy), NULL);
 	vbox = gtk_vbox_new(FALSE, 10);
 	gtk_container_add(GTK_CONTAINER(wlw), vbox);
 	gtk_container_border_width(GTK_CONTAINER(vbox), 5);
@@ -657,16 +694,20 @@ win_list_show(GtkWidget *wgt, gpointer cbdata)
 	wlw_data = gtk_clist_new_with_titles(WIN_LIST_NUM_COLUMNS, titles);
 	gtk_clist_set_column_width(GTK_CLIST(wlw_data), FnumCol, 25);
 	gtk_clist_set_column_justification(GTK_CLIST(wlw_data),
-		FnumCol, GTK_JUSTIFY_LEFT);
+					   FnumCol, GTK_JUSTIFY_LEFT);
 	gtk_clist_set_column_width(GTK_CLIST(wlw_data), FsizeCol, 55);
 	gtk_clist_set_column_justification(GTK_CLIST(wlw_data),
-		FsizeCol, GTK_JUSTIFY_CENTER);
+					   FsizeCol, GTK_JUSTIFY_CENTER);
 	gtk_clist_column_titles_passive(GTK_CLIST(wlw_data));
-	gtk_clist_set_policy(GTK_CLIST(wlw_data),
-		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), wlw_data, TRUE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(wlw_data),
-		"select_row", GTK_SIGNAL_FUNC(win_list_select), NULL);
+	scrolled_win = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scrolled_win), wlw_data);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
+	gtk_widget_show(scrolled_win);
+	gtk_signal_connect(GTK_OBJECT(wlw_data), "select_row",
+			   GTK_SIGNAL_FUNC(win_list_select), NULL);
 
 	/* for each window, add info into the clist */
 	wlp = winlist;
@@ -684,17 +725,17 @@ win_list_show(GtkWidget *wgt, gpointer cbdata)
 
 	tmp = gtk_button_new_with_label("Close Selected");
 	gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		GTK_SIGNAL_FUNC(win_list_close_selected), NULL);
+			   GTK_SIGNAL_FUNC(win_list_close_selected), NULL);
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 2);
 
 	tmp = gtk_button_new_with_label("Close All");
 	gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		GTK_SIGNAL_FUNC(win_close_all_cb), NULL);
+			   GTK_SIGNAL_FUNC(win_close_all_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 2);
 
 	tmp = gtk_button_new_with_label("Exit Window List");
 	gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-			GTK_SIGNAL_FUNC(win_list_destroy), NULL);
+			   GTK_SIGNAL_FUNC(win_list_destroy), NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 2);
 	GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default(tmp);
@@ -733,7 +774,7 @@ win_list_destroy(GtkWidget *wgt, gpointer cbdata)
 static void
 win_list_close_selected(GtkWidget *wgt, gpointer cbdata)
 {
-	GList *wlp;
+	GSList *wlp;
 	win_t *w;
 
 	if (wlw == NULL)
@@ -762,7 +803,7 @@ win_list_remove(win_t *w)
 	char buf[4];
 	int i;
 	GtkCList *clist;
-	GList *wlp;
+	GSList *wlp;
 	win_t *wp;
 	
 	if (wlw == NULL)
@@ -845,11 +886,11 @@ win_list_add(win_t *w, char *title)
  *
  * 2. raises the window which was selected.
  */
-static void
+static int
 win_list_select(GtkWidget *wgt, unsigned row, int column,
 	GdkEventButton *event, gpointer cbdata)
 {
-	GList *wlp;
+	GSList *wlp;
 	win_t *w;
 
 	curwinrow = row;
@@ -862,6 +903,8 @@ win_list_select(GtkWidget *wgt, unsigned row, int column,
 		}
 		wlp = wlp->next;
 	}
+
+	return TRUE;
 } /* win_list_select */
 
 
